﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;

using System.Threading;
using System.IO;

namespace TreeD3D
{
    public partial class MainForm : Form
    {
        TIcon icon = null;

        public MainForm()
        {
            InitializeComponent();
            DisableEditing();

            icon = new TIcon();
            ChangeIcon();
            EnableEditing();
            RefreshAll();
        }   

        // PRESS KEY ON THE FORM
        void MainFormKeyDown(object sender, KeyEventArgs e)
        {

        }
        
        
#region BASIC

        //ENABLE TO EDIT ALL DATA
        void EnableEditing()
        {
            foreach (ToolStripItem C in toolMenu.Items)
                C.Enabled = true;
            panelColors.Enabled = true;
            panelSide.Enabled = true;
            panelTop.Enabled = true;
            panelOutput.Enabled = true;
            tabControl1.Enabled = true;
        }

        //DISABLE EDITING
        void DisableEditing()
        {
            foreach (ToolStripItem C in toolMenu.Items)
                C.Enabled = false;
            butNew.Enabled = true;
            butOPen.Enabled = true;
            panelColors.Enabled = false;
            panelSide.Enabled = false;
            panelTop.Enabled = false;
            panelOutput.Enabled = false;
            tabControl1.Enabled = false;
        }
        
        //REFRESH PANELS
        void RefreshAll()
        {
            panelColors.Refresh();
            panelSide.Refresh();
            panelTop.Refresh();
            panelOutput.Refresh();
        }

        //CHANGE ICON
        void ChangeIcon()
        {
           // if (icon == null) return;
            icon.histEngine.Init(butHistoryList);

            int t = icon.sizeZ - 1 - icon.levelTop;
            int s = icon.levelSide;

            scrollBarSide.Maximum = icon.sizeY - 1;
            scrollBarTop.Maximum = icon.sizeZ - 1;

            if (t > scrollBarTop.Maximum) t = scrollBarTop.Maximum;
            if (t < scrollBarTop.Minimum) t = scrollBarTop.Minimum;
            if (s > scrollBarSide.Maximum) s = scrollBarSide.Maximum;
            if (s < scrollBarSide.Minimum) s = scrollBarSide.Minimum;

            scrollBarSide.Value = s;
            scrollBarTop.Value = t;

            icon.palleteX = panelColors.Width * 1.0f / icon.palleteXLenght;
            icon.palleteY = panelColors.Height * 1.0f / icon.palleteYLenght;

            icon.panelDimensionTopX = panelTop.Width - 32;
            icon.panelDimensionTopY = panelTop.Height;
            icon.panelDimensionSideX = panelSide.Width - 32;
            icon.panelDimensionSideY = panelSide.Height;

            icon.ResizeOutputBitmap();
        }

#endregion

#region TOOLBAR

        private void butShoowTabControl_Click(object sender, EventArgs e)
        {
            butShoowTabControl.Checked = !butShoowTabControl.Checked;
            tabControl1.Visible = butShoowTabControl.Checked;
        }

        void butExit2_Click(object sender, EventArgs e)
        {
            Close();
        }

        void ButHelpClick(object sender, EventArgs e)
        {
            // TO DO
        }

        void ButAboutClick(object sender, EventArgs e)
        {
            AboutBox1 SS = new AboutBox1();
            SS.ShowDialog();
        }

        void ButSaveAs2Click(object sender, EventArgs e)
        {
            if (icon == null) return;
            saveFileDialog1.InitialDirectory = Directory.GetCurrentDirectory();
            saveFileDialog1.Filter = "Bladum icons package|*.dat";
            saveFileDialog1.FileName = icon.currentFile;
            if (saveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                icon.Save2(saveFileDialog1.FileName);
                icon.currentFile = saveFileDialog1.FileName;
            }
        }

        void ButClose2Click(object sender, EventArgs e)
        {
            if (icon == null) return;
            DisableEditing();
            icon = null;
            RefreshAll();
        }

        void ButPrint2Click(object sender, EventArgs e)
        {

        }

        void ButRecentClick(object sender, EventArgs e)
        {
            // TO DO
        }
        

        void ButNewClick(object sender, EventArgs e)
        {
            icon = new TIcon();
            ChangeIcon();
            EnableEditing();
            RefreshAll();
        }
        
        void ButOPenClick(object sender, EventArgs e)
        {
            openFileDialog1.InitialDirectory = Directory.GetCurrentDirectory();
            openFileDialog1.Filter = "Bladum icons package|*.dat";
            if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                icon = TIcon.Load2(openFileDialog1.FileName);
                if (icon == null) return;
                icon.currentFile = openFileDialog1.FileName;
                ChangeIcon();
                EnableEditing();
                Refresh();
            }
        }
        
        void ButSaveClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.Save2();
        }
                
        void ButCutClick(object sender, EventArgs e)
        {

        }
        
        void ButCopyClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            Clipboard.Clear();
            Clipboard.SetData("int[,,]", icon.data);
        }
        
        void ButPasteClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            if (Clipboard.ContainsData("int[,,]") == false) return;

            icon.data = (int[, ,])Clipboard.GetData("int[,,]");
            RefreshAll();
        }
        
        void ButUndoClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.Undo();
            RefreshAll();
        }
        
        void ButRedoClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.Redo();
            RefreshAll();
        }
        
        void ButZoomInClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.ChangeZoom(1);
            panelOutput.Refresh();
        }
        
        void ButZoomOutClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.ChangeZoom(-1);
            panelOutput.Refresh();
        }
        
        void ButPen01Click(object sender, EventArgs e)
        {
            if (icon == null) return;
            ToolStripButton PP = (ToolStripButton)sender;

            int sel = Int16.Parse(PP.Tag.ToString());
            butPen01.Checked = false;
            butPen02.Checked = false;
            butPen03.Checked = false;
            butPen04.Checked = false;
            butPen05.Checked = false;
            butPen06.Checked = false;
            butPen08.Checked = false;
            butPen09.Checked = false;
            butPen10.Checked = false;
            butPen11.Checked = false;
            butPen12.Checked = false;
            butPen13.Checked = false;
            butPen14.Checked = false;
            butPen15.Checked = false;
            butPen16.Checked = false;
            butPen17.Checked = false;
            butPen18.Checked = false;
            butPen19.Checked = false;

            icon.drawType = sel;
            PP.Checked = true;

            RefreshAll();
        }

#endregion

#region PANELS

        int selectedColor = 4;

        private void panelTop_MouseUp(object sender, MouseEventArgs e)
        {
            // WHAT ??
        }

        private void panelSide_MouseUp(object sender, MouseEventArgs e)
        {
            // WHAT ??
        }

        void PanelTopPaint(object sender, PaintEventArgs e)
        {
            if (icon == null) return;
            icon.DrawTop(e.Graphics);
        }
        
        void PanelSidePaint(object sender, PaintEventArgs e)
        {
            if (icon == null) return;
            icon.DrawSide(e.Graphics);
        }
        
        void PanelOutputPaint(object sender, PaintEventArgs e)
        {
            if (icon == null) return;
            icon.DrawOnScreen(e.Graphics);
        }
        
        void PanelColorsPaint(object sender, PaintEventArgs e)
        {
            if (icon == null) return;
            icon.DrawColorsOnPanel(e.Graphics, selectedColor);
        }
        
        void ScrollBarTopScroll(object sender, ScrollEventArgs e)
        {
            if (icon == null) return;
            icon.levelTop = icon.sizeZ - 1 - scrollBarTop.Value;
            RefreshAll();
        }
        
        void ScrollBarSideScroll(object sender, ScrollEventArgs e)
        {
            if (icon == null) return;
            icon.levelSide = scrollBarSide.Value;
            RefreshAll();
        }
        
        void PanelColorsMouseDown(object sender, MouseEventArgs e)
        {
            if (icon == null) return;
            selectedColor = (int)(e.X / (int)icon.palleteX + (e.Y / (int)icon.palleteY) * icon.palleteXLenght);
            if (selectedColor > 255) selectedColor = 0;
            RefreshAll();
        }
        
        void PanelSideMouseDown(object sender, MouseEventArgs e)
        {
            if (icon == null) return;
            int sss = icon.ClickSide(e, selectedColor, true);
            if (sss < 9999)
            {
                selectedColor = sss;
                RefreshAll();
            }
        }
        
        void PanelTopMouseDown(object sender, MouseEventArgs e)
        {
            if (icon == null) return;
            int sss = icon.ClickTop(e, selectedColor, true);
            if (sss < 9999)
            {
                selectedColor = sss;
                RefreshAll();
            }
        }
        
                
        void PanelOutputMouseDown(object sender, MouseEventArgs e)
        {
            if (icon == null) return;
            if (e.Button == MouseButtons.Right)
            {
                icon.ChangeOffset(e, panelOutput.Size);
                RefreshAll();
            }
        }
        
        void PanelTopMouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.None) return;

            if (icon == null) return;
            int sss = icon.ClickTop(e, selectedColor, false);
            if (sss < 9999)
            {
                selectedColor = sss;
                RefreshAll();
            }
        }
        
        void PanelSideMouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.None) return;

            if (icon == null) return;
            int sss = icon.ClickSide(e, selectedColor, false);
            if (sss < 9999)
            {
                selectedColor = sss;
                RefreshAll();
            }
        }
        
                
        void PanelColorsResize(object sender, EventArgs e)
        {
            if (icon == null) return;

            icon.palleteX = panelColors.Width * 1.0f / icon.palleteXLenght;
            icon.palleteY = panelColors.Height * 1.0f / icon.palleteYLenght;

            RefreshAll();
        }
        
        void PanelTopResize(object sender, EventArgs e)
        {
            if (icon == null) return;

            icon.panelDimensionTopX = panelTop.Width - 32;
            icon.panelDimensionTopY = panelTop.Height;

            RefreshAll();
        }
        
        void PanelSideResize(object sender, EventArgs e)
        {
            if (icon == null) return;

            icon.panelDimensionSideX = panelSide.Width - 32;
            icon.panelDimensionSideY = panelSide.Height;

            Refresh();
        }
        
        void PanelOutputResize(object sender, EventArgs e)
        {
        	// WHAT ??
        }
#endregion
             
#region EDIT

        void ButProcessMirrorXClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.MirrorX();
            RefreshAll();
        }
        
        void ButProcessMirrorYClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.MirrorY();
            RefreshAll();
        }
        
        void ButProcessMirrorZClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.MirrorZ();
            RefreshAll();
        }
        
        void ButProcessShiftUpClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.MoveUp();
            RefreshAll();
        }
        
        void ButProcessShiftDownClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.MoveDown();
            RefreshAll();
        }
        
        void ButProcessAutoCycleCheckedChanged(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.autoCycle = butProcessAutoCycle.Checked;
        }
        
        void ButProcessAutoResizeCheckedChanged(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.autoResize = butProcessAutoResize.Checked;
        }

        
        void ButProcessMoveUpClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.MoveBack();
            RefreshAll();
        }
        
        void ButProcessMoveLeftClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.MoveLeft();
            RefreshAll();
        }
        
        void ButProcessMoveRightClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.MoveRight();
            RefreshAll();
        }
        
        void ButProcessMoveDownClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.MoveFront();
            RefreshAll();
        }
        
        void ButProcessRotateZLeftClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.RotateLeft();
            ChangeIcon();
            RefreshAll();
        }
        
        void ButProcessRotateZRightClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.RotateRight();
            ChangeIcon();
            RefreshAll();
        }
        
        void ButProcessRotateYLeftClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.CycleLeft();
            ChangeIcon();
            Refresh();
        }
        
        void ButProcessRotateYRightClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.CycleRight();
            ChangeIcon();
            Refresh();
        }
        
        void ButProcessRotatXLeftClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.CycleFront();
            ChangeIcon();
            Refresh();
        }
        
        void ButProcessRotateXRightClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.CycleBack();
            ChangeIcon();
            Refresh();
        }
        
        void ButEffectLightDownClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.LightsDown();
            Refresh();
        }
        
        void ButEffectLightUpClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.LightsUp();
            Refresh();
        }
        
        void ButEffectContrastDownClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.ContrastDown();
            Refresh();
        }
        
        void ButEffectContrastUpClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.ContrastUp();
            Refresh();
        }
        
        void ButEffectHueDownClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.ColorPrev();
            ChangeIcon();
            RefreshAll();
        }
        
        void ButEffectHueUpClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.ColorNext();
            ChangeIcon();
            RefreshAll();
        }
        
        void ButEffectBlackWhiteClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.BlackAndWhite();
            Refresh();
        }
        
        void ButEffectPosterizeClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.Posterize();
            Refresh();
        }
        
        void ButEffectInverseClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.Invert();
            Refresh();
        }
        
        void ButOutputEnvelopeCheckedChanged(object sender, EventArgs e)
        {
            icon.enveloperMode = butOutputEnvelope.Checked;
            Refresh();
        }
        
        void ButOutputGlowCheckedChanged(object sender, EventArgs e)
        {
            icon.glowMode = butOutputGlow.Checked;
            Refresh();
        }

        void ButOutputAddCheckedChanged(object sender, EventArgs e)
        {
            icon.addType = butOutputAdd.Checked;
        }
        
        void ButOutputRemoveCheckedChanged(object sender, EventArgs e)
        {

            icon.removeType = butOutputRemove.Checked;
        }
        
        void ButOutputRandomCheckedChanged(object sender, EventArgs e)
        {
            icon.randomType = butOutputRandom.Checked;
        }
        
        void ButOutputDeepCheckedChanged(object sender, EventArgs e)
        {
            icon.deepMode = butOutputDeep.Checked;
            icon.deepLevel = (int)butOutputDeepLevel.Value; 
        }
        
        void ButOutputAlphaCheckedChanged(object sender, EventArgs e)
        {
            icon.alphaMode = butOutputAlpha.Checked;
        }
        
        void ButOutputShadowCheckedChanged(object sender, EventArgs e)
        {
            icon.upperShadow = butOutputShadow.Checked;
            icon._colorShadowLevel = (int)butOutputShadowLevel.Value;
            RefreshAll();
        }
        
        void ButOutputEnvelopeAlphaValueChanged(object sender, EventArgs e)
        {
            icon._borderAlpha = (int)butOutputEnvelopeAlpha.Value*10;
            Refresh();
        }
        
        void ButOutputEnvelopeWidthValueChanged(object sender, EventArgs e)
        {
            icon._borderSize = (int)butOutputEnvelopeWidth.Value;
            Refresh();
        }
        
        void ButOutputGlowAlphaValueChanged(object sender, EventArgs e)
        {
            icon._glowAlpha = (int)butOutputGlowAlpha.Value*10;
            Refresh();
        }
     
        void ButOutputGlowWidthValueChanged(object sender, EventArgs e)
        {
            icon._glowSize = (int)butOutputGlowWidth.Value;
            Refresh();
        }
        
        void ButOutputAddLevelValueChanged(object sender, EventArgs e)
        {
            icon._addChance = (int)butOutputAddLevel.Value*10;
        }
        
        void ButOutputRemoveLevelValueChanged(object sender, EventArgs e)
        {
            icon._removeChance = (int)butOutputRemoveLevel.Value*10;
        }
        
        void ButOutputRandomLevelValueChanged(object sender, EventArgs e)
        {
            icon._noiseChance = (int)butOutputRandomLevel.Value*10;
        }
        
        void ButOutputDeepLevelValueChanged(object sender, EventArgs e)
        {
            icon.deepLevel = (int)butOutputDeepLevel.Value;
        }
        
        void ButOutputAlphaLevelValueChanged(object sender, EventArgs e)
        {
            icon._glowAlpha = (int)butOutputAlphaLevel.Value*10;
        }
        
        void ButOutputShadowLevelValueChanged(object sender, EventArgs e)
        {
            icon._sizeForShadow = 100 + (int)butOutputShadowLevel.Value * 10;
        }
        
        
#endregion        
        
#region VIEW

        void buttonPaletteRefresh_Click(object sender, EventArgs e)
        {
            RefreshAll();
        }

	    void ButLevelBelowCheckedChanged(object sender, EventArgs e)
        {
            if (icon == null) return;

            RadioButton PP = (RadioButton)sender;
            int sel = Int32.Parse(PP.Tag.ToString());

            icon.onionLevel = sel;

            RefreshAll();
        }
        
        void ButViewShow02CheckedChanged(object sender, EventArgs e)
        {
            if (icon == null) return;

            RadioButton PP = (RadioButton)sender;
            int sel = Int32.Parse(PP.Tag.ToString());

            icon.iconRotate = sel;

            RefreshAll();
        }
        
        void ButViewZoomInClick(object sender, EventArgs e)
        {
            ButZoomInClick(sender, e);
        }
        
        void ButViewZoomOutClick(object sender, EventArgs e)
        {
            ButZoomOutClick(sender, e);
        }
        
        void ButViewZoomClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.ChangeZoom(0);
            panelOutput.Refresh();
        }


        int selected = 0;
        int sss = 26;

        void ButPalettePickClick(object sender, EventArgs e)
        {
            if (icon == null) return;

            if (colorDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                Color CC = colorDialog1.Color;

                butPaletteR.Value = CC.R;
                butPaletteG.Value = CC.G;
                butPaletteB.Value = CC.B;
                panelPalette.Refresh();
            }
        }
        
        void ButPaletteExportClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            saveFileDialog1.InitialDirectory = Directory.GetCurrentDirectory();
            saveFileDialog1.Filter = "Palette files|*.pal";
            if (saveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                icon.ExportPalete(saveFileDialog1.FileName);
            }
        }
        
        void ButPaletteImportClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            openFileDialog1.InitialDirectory = Directory.GetCurrentDirectory();
            openFileDialog1.Filter = "Palette files|*.pal";
            if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                icon.ImportPalete(openFileDialog1.FileName);
                icon.UpdateColorShadowTable();
                panelPalette.Refresh();
            }
        }
        
        void ButPaletteRValueChanged(object sender, EventArgs e)
        {
            TColor C = icon.colors1[selected * 16];
            icon.colorsBase[selected * 16].R = (byte)butPaletteR.Value;

            icon.UpdateColorShadowTable();
            panelPalette.Refresh();
        }
        
        void ButPaletteGValueChanged(object sender, EventArgs e)
        {
            TColor C = icon.colors1[selected * 16];
            icon.colorsBase[selected * 16].G = (byte)butPaletteG.Value;

            icon.UpdateColorShadowTable();
            panelPalette.Refresh();
        }
        
        void ButPaletteBValueChanged(object sender, EventArgs e)
        {
            TColor C = icon.colors1[selected * 16];
            icon.colorsBase[selected * 16].B = (byte)butPaletteB.Value;

            icon.UpdateColorShadowTable();
            panelPalette.Refresh();
        }
        
        void PanelPaletteMouseDown(object sender, MouseEventArgs e)
        {
            if (icon == null) return;

            SaveColor();
            selected = (e.X / sss) + (e.Y / sss) * 4;
            LoadColor();
            panelPalette.Refresh();
        }

        void LoadColor()
        {
            butPaletteR.Value = icon.colorsBase[selected * 16].R;
            butPaletteG.Value = icon.colorsBase[selected * 16].G;
            butPaletteB.Value = icon.colorsBase[selected * 16].B;
        }
        void SaveColor()
        {
            icon.colorsBase[selected * 16] = new TColor((byte)butPaletteR.Value, (byte)butPaletteG.Value, (byte)butPaletteB.Value);
            icon.UpdateColorShadowTable();
        }
        
        void PanelPalettePaint(object sender, PaintEventArgs e)
        {
            if (icon == null) return;

            SolidBrush SS = new SolidBrush(Color.Black);


            SS.Color = icon.colorsBase[0].ToColor();
            e.Graphics.FillRectangle(SS, 0, 0, sss, sss);
            SS.Color = icon.colorsBase[16].ToColor();
            e.Graphics.FillRectangle(SS, sss, 0, sss, sss);
            SS.Color = icon.colorsBase[32].ToColor();
            e.Graphics.FillRectangle(SS, sss*2, 0, sss, sss);
            SS.Color = icon.colorsBase[48].ToColor();
            e.Graphics.FillRectangle(SS, sss*3, 0, sss, sss);

            SS.Color = icon.colorsBase[64].ToColor();
            e.Graphics.FillRectangle(SS, 0, sss, sss, sss);
            SS.Color = icon.colorsBase[80].ToColor();
            e.Graphics.FillRectangle(SS, sss, sss, sss, sss);
            SS.Color = icon.colorsBase[96].ToColor();
            e.Graphics.FillRectangle(SS, sss*2, sss, sss, sss);
            SS.Color = icon.colorsBase[112].ToColor();
            e.Graphics.FillRectangle(SS, sss*3, sss, sss, sss);

            SS.Color = icon.colorsBase[128].ToColor();
            e.Graphics.FillRectangle(SS, 0, sss*2, sss, sss);
            SS.Color = icon.colorsBase[144].ToColor();
            e.Graphics.FillRectangle(SS, sss, sss*2, sss, sss);
            SS.Color = icon.colorsBase[160].ToColor();
            e.Graphics.FillRectangle(SS, sss*2, sss*2, sss, sss);
            SS.Color = icon.colorsBase[176].ToColor();
            e.Graphics.FillRectangle(SS, sss*3, sss*2, sss, sss);

            SS.Color = icon.colorsBase[192].ToColor();
            e.Graphics.FillRectangle(SS, 0, sss*3, sss, sss);
            SS.Color = icon.colorsBase[208].ToColor();
            e.Graphics.FillRectangle(SS, sss, sss*3, sss, sss);
            SS.Color = icon.colorsBase[224].ToColor();
            e.Graphics.FillRectangle(SS, sss*2, sss*3, sss, sss);
            SS.Color = icon.colorsBase[240].ToColor();
            e.Graphics.FillRectangle(SS, sss*3, sss*3, sss, sss);

            e.Graphics.DrawRectangle(Pens.Black, (selected % 4) * sss, (selected / 4) * sss, sss, sss);
        }
        
        void ButLight02CheckedChanged(object sender, EventArgs e)
        {
            if (icon == null) return;
            RadioButton PP = (RadioButton)sender;

            int sel = Int16.Parse(PP.Tag.ToString());
            icon._lightSource = sel;
            icon.UpdateColorShadowTable();
            RefreshAll();
        }
        
        void ButLightBackgroundClick(object sender, EventArgs e)
        {
            if (icon == null) return;

            if (colorDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                Color CC = colorDialog1.Color;
                butLightBackground.BackColor = CC;
                icon._backGroundColor = new TColor(CC);
                icon.UpdateColorShadowTable();
                RefreshAll();
            }
        }
        
        void ButLightMarkerCheckedChanged(object sender, EventArgs e)
        {
            icon.drawMarkersOnScreen = butLightMarker.Checked;
            RefreshAll();
        }
        
        void ButLightOutCheckedChanged(object sender, EventArgs e)
        {
            icon.drawRealView = butLightOut.Checked;
            RefreshAll();
        }

#endregion
        
#region MISC

        private void butHistoryClear_Click(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.ClearHistory();
            RefreshAll();
        }
        
        void ButHistoryUndoAllClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.UndoAll();
            RefreshAll();
        }
        
        void ButHistoryUndoClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.Undo();
            RefreshAll();
        }
        
        void ButHistoryRedoClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.Redo();
            RefreshAll();
        }
        
        void ButHistoryRedoAllClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon.RedoAll();
            RefreshAll();
        }

        void ButHistoryListSelectedIndexChanged(object sender, EventArgs e)
        {
            int x = butHistoryList.SelectedIndex;
            if (x < 0) return;

            //TO DO 

            RefreshAll();
        }

        void ButExportView02CheckedChanged(object sender, EventArgs e)
        {
            //WHAT IS ??
        }

        void ButExportAllClick(object sender, EventArgs e)
        {
            butExportView01.Checked = true;
            butExportView02.Checked = true;
            butExportView03.Checked = true;
            butExportView04.Checked = true;
            butExportView05.Checked = true;
            butExportView06.Checked = true;
            butExportView07.Checked = true;
            butExportView08.Checked = true;
        }       
        
        void ButExportSquareClick(object sender, EventArgs e)
        {
            butExportView01.Checked = false;
            butExportView02.Checked = true;
            butExportView03.Checked = false;
            butExportView04.Checked = true;
            butExportView05.Checked = false;
            butExportView06.Checked = true;
            butExportView07.Checked = false;
            butExportView08.Checked = true;
        }
        
        void ButExportIsoClick(object sender, EventArgs e)
        {
            butExportView01.Checked = true;
            butExportView02.Checked = false;
            butExportView03.Checked = true;
            butExportView04.Checked = false;
            butExportView05.Checked = true;
            butExportView06.Checked = false;
            butExportView07.Checked = true;
            butExportView08.Checked = false;        	
        }
       
        void ButExportMultiCheckedChanged(object sender, EventArgs e)
        {
            //WHAT HERE ??	
        }

        private void butExportHD_CheckedChanged(object sender, EventArgs e)
        {
            //WHAT HERE ??
        }

        void ButExportPrintClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            saveFileDialog1.InitialDirectory = Directory.GetCurrentDirectory();
            saveFileDialog1.Filter = "Png graphic files|*.png";
            saveFileDialog1.FileName = "output.png";
            if (saveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                int[] views = new int[9];

                views[0] = Int16.Parse(butExportView01.Tag.ToString());
                views[1] = Int16.Parse(butExportView02.Tag.ToString());
                views[2] = Int16.Parse(butExportView03.Tag.ToString());
                views[3] = Int16.Parse(butExportView04.Tag.ToString());
                views[4] = Int16.Parse(butExportView05.Tag.ToString());
                views[5] = Int16.Parse(butExportView06.Tag.ToString());
                views[6] = Int16.Parse(butExportView07.Tag.ToString());
                views[7] = Int16.Parse(butExportView08.Tag.ToString());

                icon.DrawAllSaveIntoFiles(saveFileDialog1.FileName, butExportHD.Checked, !butExportMulti.Checked, views);
            }
        }

        void ButExportScaleValueChanged(object sender, EventArgs e)
        {
            if (icon == null) return;
            icon._outPutSize = (int)butExportScale.Value;
        }
        
        void ButExportObjClick(object sender, EventArgs e)
        {
            // TO DO
        }
        
        void ButExportClipboardClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            Clipboard.Clear();
            if (icon.bmpOut == null) return;
            Clipboard.SetImage(icon.bmpOut);
        }

        string method = "5";

        void ButSizeAnchor02CheckedChanged(object sender, EventArgs e)
        {
            if (icon == null) return;
            RadioButton RR = (RadioButton)sender;
            method = RR.Tag.ToString();        	
        }
        
        void ButSizeResizeClick(object sender, EventArgs e)
        {
            if (icon == null) return;
            int difX = (int)butSizeX.Value;
            int difY = (int)butSizeY.Value;
            int difZ = (int)butSizeZ.Value;
            icon.ResizeData(difX, difY, difZ, method);
            ChangeIcon();
            RefreshAll();
        }
        
        void ButSizeCanvasClick(object sender, EventArgs e)
        {
            if (icon == null) return;

            // TO DO CHANGE CANVAS ONLY

            ChangeIcon();
            RefreshAll();
        }
        
        void ButSizeXValueChanged(object sender, EventArgs e)
        {
        	// WHAT HERE ?
        }
        
        void ButSizeYValueChanged(object sender, EventArgs e)
        {
            // WHAT HERE ?
        }
        
        void ButSizeZValueChanged(object sender, EventArgs e)
        {
            // WHAT HERE ?
        }
#endregion

#region TOOLS

       
        void ButLayerAddClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerCloneClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerRemoveClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerMergeClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerListSelectedIndexChanged(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerUpClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerDownClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerMirrorYClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerMirrorXClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerMirrorZClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerCopyClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerSumClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerInvertClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerAndClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerSubtractClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButLayerClearClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButAnimAddClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButAnimCloneClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButAnimDeleteClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButAnimListSelectedIndexChanged(object sender, EventArgs e)
        {
        	
        }
        
        void ButAnimUpClick(object sender, EventArgs e)
        {
        	
        }
        
        void ButAnimDownClick(object sender, EventArgs e)
        {
        	
        }
        
        
#endregion

        
    }
}
