﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;

namespace TreeD3D
{    
    public class TColor
    {
        public int A = 255;
        public int R = 0;
        public int G = 0;
        public int B = 0;

        public TColor(int _r, int _g, int _b)
        {
            R = _r;
            G = _g;
            B = _b;
        }

        public TColor(int _r, int _g, int _b, int _a)
        {
            A = _a;
            R = _r;
            G = _g;
            B = _b;
        }

        public TColor(float _r, float _g, float _b)
        {
            R = (int)_r;
            G = (int)_g;
            B = (int)_b;
        }


        public TColor(Color cc)
        {
            R = cc.R;
            G = cc.G;
            B = cc.B;
            A = cc.A;
        }

        public Color ToColor()
        {
            return Color.FromArgb(A, R, G, B);
        }
    }

    public class TIconHistory
    {
        public string desc = "";
        public int[, ,] data = new int[3, 3, 3];
    }

    public class TIconHistoryEngine
    {
        public List<TIconHistory> history = new List<TIconHistory>();
        public ListBox listBox = null;
        public int selectedID = 0;

        public void Init(ListBox pp)
        {
            listBox = pp;
            pp.Items.Clear();            
            AddEvent("New Icon", new int[12, 12, 12]);
        }

        public void AddEvent(string name, int[, ,] data)
        {
            TIconHistory TT = new TIconHistory();
            TT.desc = name;
            TT.data = data;
            history.Insert(0,TT);
            selectedID = 0;
            listBox.Items.Insert(0, name);
            listBox.SelectedIndex = 0;
            listBox.Refresh();
        }

        public TIconHistory UndoEvent()
        {
            selectedID++;
            if (selectedID >= history.Count) selectedID = history.Count - 1;
            listBox.SelectedIndex = selectedID;
            listBox.Refresh();
            return history[selectedID];
        }

        public TIconHistory RedoEvent()
        {
            selectedID--;
            if (selectedID < 0) selectedID = 0;
            listBox.SelectedIndex = selectedID;
            listBox.Refresh();
            return history[selectedID];
        }

        public TIconHistory UndoAllEvents()
        {
            selectedID = history.Count-1;
            listBox.SelectedIndex = selectedID;
            listBox.Refresh();
            return history[selectedID];
        }

        public TIconHistory RedoEventAll()
        {
            selectedID = 0;
            listBox.SelectedIndex = selectedID;
            listBox.Refresh();
            return history[selectedID];
        }

        public void ClearHistory()
        {
            history.Clear();
            listBox.Items.Clear();            
            AddEvent("New Icon", new int[12, 12, 12]);
            selectedID = 0;
            listBox.SelectedIndex = selectedID;
            listBox.Refresh();
        }
    }
            
    public class TIcon
    {
        //CONSTRUCTOR
        public TIcon()
        {
            ResizeOutputBitmap();

            //create main data container
            data = new int[sizeX, sizeY, sizeZ];

            //create colors
            colorsBase = new TColor[256];
            //colors for each side of the rectangle
            colors1 = new TColor[256];
            colors2 = new TColor[256];
            colors3 = new TColor[256];

            //current file name
            currentFile = "new.dat";

            CreateNonSerialized();
            CreateStandardColorSet();
            UpdateColorShadowTable();
        }

#region data

        //history engine
        public TIconHistoryEngine histEngine = new TIconHistoryEngine();

        static Random RR = new Random();

        public string name = "icon";
        public string currentFile = "";

        //main pixel data
        public int[,,] data;

        //indicator for selected color for LMB and RMB
        public int selectedColor = 4;
        public int selectedColor2 = 4;

        //size of colorItem in colorBox
        public float palleteX = 10;
        public float palleteY = 10;
        //number of colors in colorBox
        public int palleteXLenght = 64;
        public int palleteYLenght = 4;

        //some offset for moving output screen
        public int outPutOffsetX = 0;
        public int outPutOffsetY = 0;

        //base size of pixel data
        public int sizeX = 12;
        public int sizeY = 12;
        public int sizeZ = 12;

        //size of the single output graphic
        public int bmpX = 48;
        public int bmpY = 60;

        //current view level on top and side level
        public int levelTop = 0;
        public int levelSide = 0;

        //current tool
        public int drawType = 0;

        //some general flags
        public int onionLevel = 0;        
        public bool randomType = false;
        public bool removeType = false;
        public bool addType = false;
        public bool alphaMode = false;
        public bool deepMode = false;
        public int deepLevel = 0;
        public bool glowMode = false;
        public bool enveloperMode = true;
        public bool upperShadow = false;
        public bool drawRealView = false;
        public bool drawMarkersOnScreen = false;
        public bool autoCycle = false;
        public bool autoResize = false;

        //some zoom stuff
        static int zoomLevelBegin = 3;
        public int zoomLevel = zoomLevelBegin;
        public int iconRotate = 0;
        public int[] zoomLevelValues = { 1, 2, 3, 4, 5, 6, 7, 8 };
        int GetZoomLevel
        {
            get
            {
                return zoomLevelValues[zoomLevel];
            }
        }
        public int zoomLevelValueMax = 7;

        // ??? wtf ??
        public int iconSetSelect = 0;
        public int iconSelect = 0;

        //some data about palette
        public string pallHeader = "JASC-PAL\n0100\n256\n";
        public int panelDimensionTopX = 108;
        public int panelDimensionTopY = 108;
        public int panelDimensionSideX = 108;
        public int panelDimensionSideY = 108;

        //base color table
        public TColor[] colorsBase;

        //colors for each side of the box  TOP / LEFT / RIGHT
        public TColor[] colors1;
        public TColor[] colors2;
        public TColor[] colors3;

        //data for mouse moving events
        public int lastMouseTopX = 0;
        public int lastMouseTopY = 0;
        public int lastMouseSideX = 0;
        public int lastMouseSideY = 0;

        //just some help stuff
        [NonSerialized]
        public Brush backGroundSolid = Brushes.Black;
        [NonSerialized]
        public Bitmap bmpOut;

        //background color
        public TColor _backGroundColor = new TColor(0, 0, 0, 0);

        //some shadow data
        public int _colorShadowLevel = 88;
        public int _shadowTop = 100;
        public int _shadowLeft = 75;
        public int _shadowRight = 45;

        //other properties
        public int _lightSource = 0;
        public int _alpha = 60;
        public int _noiseChance = 20;
        public int _addChance = 20;
        public int _removeChance = 20;
        public int _borderSize = 4;
        public int _borderAlpha = 60;
        public int _glowSize = 10;
        public int _glowAlpha = 30;
        public int _outPutSize = 25;
        public int _sizeForShadow = 150;
        
        //color of the marker on the output
        public TColor _markerColor = new TColor(90, 90, 90, 90);

        //shadow table for light direction, 3 sides 
        int[] shadowTableTop = { 90, 100, 90, 80, 70, 60, 70, 80 };
        int[] shadowTableLeft = { 85, 70, 55, 40, 55, 70, 85, 100 };
        int[] shadowTableRight = { 40, 60, 80, 100, 80, 60, 40, 20 };

#endregion

#region colors

        //CREATE NON SERIALIZED
        public void CreateNonSerialized()
        {
            bmpOut = new Bitmap(bmpX, bmpY);
        }

        //CREATE BASE COLOR
        public void CreateStandardColorSet()
        {
            CreateSubColor(0, 250, 250, 250);
            CreateSubColor(16, 250, 200, 0);
            CreateSubColor(32, 250, 125, 125);
            CreateSubColor(48, 150, 225, 125);
            CreateSubColor(64, 200, 225, 100);
            CreateSubColor(80, 250, 200, 250);
            CreateSubColor(96, 250, 200, 100);
            CreateSubColor(112, 150, 175, 200);
            CreateSubColor(128, 175, 200, 250);
            CreateSubColor(144, 250, 250, 125);
            CreateSubColor(160, 175, 150, 75);
            CreateSubColor(176, 250, 225, 200);
            CreateSubColor(192, 225, 200, 250);
            CreateSubColor(208, 75, 200, 250);
            CreateSubColor(224, 225, 225, 250);
            CreateSubColor(240, 125, 125, 150);

            UpdateColorShadowTable();
        }

        //CREATE SUB COLOR
        void CreateSubColor(int i, int _R, int _G, int _B)
        {
            float tt = 1.0f;
            for (int nn = 0; nn < 16; nn++)
            {
                if (nn + i > 255) continue;

                int R = (int)(_R * tt);
                int G = (int)(_G * tt);
                int B = (int)(_B * tt);

                colorsBase[i + nn] = new TColor(R, G, B);
                tt = tt * _colorShadowLevel / 100;
            }

        }
        void CreateSubColor(int i)
        {
            float tt = 1.0f;
            TColor aaa = colorsBase[i];
            for (int nn = 0; nn < 16; nn++)
            {
                if (nn + i > 255) continue;

                int R = aaa.R;
                int G = aaa.G;
                int B = aaa.B;

                R = (int)(R * tt);
                G = (int)(G * tt);
                B = (int)(B * tt);

                colors1[i + nn] = new TColor(R * _shadowTop / 100, G * _shadowTop / 100, B * _shadowTop / 100);
                colors2[i + nn] = new TColor(R * _shadowLeft / 100, G * _shadowLeft / 100, B * _shadowLeft / 100);
                colors3[i + nn] = new TColor(R * _shadowRight / 100, G * _shadowRight / 100, B * _shadowRight / 100);

                tt = tt * _colorShadowLevel / 100;
            }

        }

        //UPDATE COLOR
        public void UpdateColorShadowTable()
        {
            _shadowTop = shadowTableTop[_lightSource];
            _shadowLeft = shadowTableLeft[_lightSource];
            _shadowRight = shadowTableRight[_lightSource];

            for (int nn = 0; nn < 16; nn++)
                CreateSubColor(nn * 16);

            backGroundSolid = new SolidBrush(_backGroundColor.ToColor());
        }

        //PROCESS PALETTE
        public void ImportPalete(string path)
        {
            string[] jj = File.ReadAllLines(path);

            int n = 0;
            int lin = 0;
            foreach (string s in jj)
            {
                lin++;
                if (s.Length < 2) continue;
                if (lin < 4) continue;

                string[] kk = s.Split(' ');
                int R = Int16.Parse(kk[0]);
                int G = Int16.Parse(kk[1]);
                int B = Int16.Parse(kk[2]);

                colorsBase[n] = new TColor(R, G, B);
                n++;
            }

            UpdateColorShadowTable();
        }
        public void ExportPalete(string path)
        {
            string jj = pallHeader;

            for (int i = 0; i < 256; i++)
                jj += colorsBase[i].R + " " + colorsBase[i].G + " " + colorsBase[i].B + "\n";

            File.WriteAllText(path, jj);
        }

        //light down color
        Color LightDownColor(Color col, int sad)
        {
            int mm = sizeZ * _sizeForShadow / 100;

            return Color.FromArgb(col.A,
                        col.R * sad / mm,
                        col.G * sad / mm,
                        col.B * sad / mm);
        }

        //more contast
        internal void ContrastUp()
        {
            histEngine.AddEvent("Constrast up", data);

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {

                    }
        }

        //less contrast
        internal void ContrastDown()
        {
            histEngine.AddEvent("Constrast down", data);

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {

                    }
        }

        //more Hue
        internal void ColorNext()
        {
            histEngine.AddEvent("Hue up", data);

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        int GG = data[x, y, z];
                        bool isAlpha = false;
                        if (GG == 0)
                        {
                            continue;
                        }
                        if (GG < 0)
                        {
                            GG = -GG;
                            isAlpha = true;
                        }
                        int SS = GG / 16;
                        int DD = GG % 16;

                        //cycle colors up
                        SS = (SS + 1) % 16;

                        //stay just major value
                        GG = SS * 16 + DD;

                        //back to alpha
                        if (isAlpha) GG = -GG;

                        data[x, y, z] = GG;
                    }
        }

        //less hue
        internal void ColorPrev()
        {
            histEngine.AddEvent("Hue down", data);

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        int GG = data[x, y, z];
                        bool isAlpha = false;
                        if (GG == 0)
                        {
                            continue;
                        }
                        if (GG < 0)
                        {
                            GG = -GG;
                            isAlpha = true;
                        }
                        int SS = GG / 16;
                        int DD = GG % 16;

                        //cycle colors down
                        SS--;
                        if (SS < 0) SS = 15;

                        //stay just major value
                        GG = SS * 16 + DD;

                        //back to alpha
                        if (isAlpha) GG = -GG;

                        data[x, y, z] = GG;
                    }
        }

        //posterize -> remove sub colors only main colors
        internal void Posterize()
        {
            histEngine.AddEvent("Posterize", data);

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        int GG = data[x, y, z];
                        bool isAlpha = false;
                        if (GG == 0)
                        {
                            continue;
                        }
                        if (GG < 0)
                        {
                            GG = -GG;
                            isAlpha = true;
                        }
                        int SS = GG / 16;
                        int DD = GG % 16;

                        //stay just major value
                        GG = SS * 16;

                        //back to alpha
                        if (isAlpha) GG = -GG;

                        data[x, y, z] = GG;
                    }
        }

        //switch into one color in the matrix
        internal void BlackAndWhite()
        {
            histEngine.AddEvent("Black & white", data);

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        int GG = data[x, y, z];
                        bool isAlpha = false;
                        if (GG == 0)
                        {
                            continue;
                        }
                        if (GG < 0)
                        {
                            GG = -GG;
                            isAlpha = true;
                        }
                        int SS = GG / 16;
                        int DD = GG % 16;

                        //turn to black and white
                        GG = DD;

                        //back to alpha
                        if (isAlpha) GG = -GG;

                        data[x, y, z] = GG;
                    }
        }

        //turn light into dark and the oposite in the range of one color
        internal void Invert()
        {
            histEngine.AddEvent("Invert", data);

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        int GG = data[x, y, z];
                        bool isAlpha = false;
                        if (GG == 0)
                        {
                            continue;
                        }
                        if (GG < 0)
                        {
                            GG = -GG;
                            isAlpha = true;
                        }
                        int SS = GG / 16;
                        int DD = GG % 16;

                        //reverse the brightness
                        GG = SS * 16 + (15 - DD);

                        //back to alpha
                        if (isAlpha) GG = -GG;

                        data[x, y, z] = GG;
                    }
        }

        // CHANGE LIGHT OF ALL PIXELS
        public void LightsDown()
        {
            histEngine.AddEvent("Light down", data);

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data[x, y, z] = LightDownSingle(data[x, y, z]);
                    }
        }
        public void LightsUp()
        {
            histEngine.AddEvent("Light up", data);

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data[x, y, z] = LightUpSingle(data[x, y, z]);
                    }
        }
        int LightDownSingle(int GG)
        {
            bool isAlpha = false;
            if (GG == 0)
            {
                return GG;
            }
            if (GG < 0)
            {
                GG = -GG;
                isAlpha = true;
            }
            int SS = GG / 16;
            int DD = GG % 16;

            //light up
            if (DD < 14) DD++;

            GG = SS * 16 + DD;

            //back to alpha
            if (isAlpha) GG = -GG;

            return GG;
        }
        int LightUpSingle(int GG)
        {
            bool isAlpha = false;
            if (GG == 0)
            {
                return GG;
            }
            if (GG < 0)
            {
                GG = -GG;
                isAlpha = true;
            }
            int SS = GG / 16;
            int DD = GG % 16;

            //light up
            if (DD > 0) DD--;

            GG = SS * 16 + DD;

            //back to alpha
            if (isAlpha) GG = -GG;

            return GG;
        }

#endregion

#region pixel processing

        // GET SINGLE PIXEL VALUE from the table
        int GetPixelCell(int x, int y, int z)
        {
            if (x < 0) return 0;
            if (y < 0) return 0;
            if (z < 0) return 0;
            if (x >= sizeX) return 0;
            if (y >= sizeY) return 0;
            if (z >= sizeZ) return 0;

            return data[x, y, z];
        }

        // GET SINGLE PIXEL VALUE from the table
        int GetPixelCell2(int[, ,] dd, int x, int y, int z)
        {
            if (x < 0) return 0;
            if (y < 0) return 0;
            if (z < 0) return 0;
            if (x >= sizeX) return 0;
            if (y >= sizeY) return 0;
            if (z >= sizeZ) return 0;

            return dd[x, y, z];
        }

        //CHANGE SIZE OF DATA
        public void ResizeData(int difX, int difY, int difZ, string method)
        {
            histEngine.AddEvent("Resize", data);

            int[, ,] data2 = new int[difX, difY, difZ];

            //TODO add here METHOD for processing anchor LEFT TOP BOTTOM RIGHT ETC

            for (int z = 0; z < difZ; z++)
                for (int x = 0; x < difX; x++)
                    for (int y = 0; y < difY; y++)
                    {
                        if (z >= difZ) continue;
                        if (x >= difX) continue;
                        if (y >= difY) continue;
                        if (z >= sizeZ) continue;
                        if (x >= sizeX) continue;
                        if (y >= sizeY) continue;

                        data2[x, y, z] = data[x, y, z];
                    }

            sizeX = difX;
            sizeY = difY;
            sizeZ = difZ;

            if (levelTop >= sizeZ) levelTop = sizeZ - 1;
            if (levelSide >= sizeY) levelSide = sizeY - 1;

            data = data2;

            ResizeOutputBitmap();
        }

        //CHANGE OUTPUT DIMENSION
        public void ResizeOutputBitmap()
        {
            bmpX = (sizeX + sizeY) * 2;
            bmpY = (sizeY + sizeX) + sizeZ * 2;
        }

        // PROCESS LOCAL COLOR (NOISE / ADD / REMOVE)
        public void Noise(int x, int y, int z, int color)
        {
            if (data[x, y, z] != 255)
                if (data[x, y, z] != 0)
                    if (data[x, y, z] == color)
                    {
                        int GG = data[x, y, z];

                        bool isAlpha = false;
                        if (GG < 0)
                        {
                            GG = -GG;
                            isAlpha = true;
                        }
                        int SS = GG / 16;
                        int DD = GG % 16;

                        int rr = RR.Next(0, 100);
                        if (rr < _noiseChance)
                            if (DD < 15) DD++;
                        if (rr >= 100 - _noiseChance)
                            if (DD > 0) DD--;

                        data[x, y, z] = SS * 16 + DD;

                        //back to alpha
                        if (isAlpha) data[x, y, z] *= -1;
                    }
        }
        public void Remove(int x, int y, int z, int color)
        {
            if (data[x, y, z] == color)
            {
                int rr = RR.Next(0, 100);
                if (rr < _removeChance)
                    data[x, y, z] = 0;
            }
        }
        public void Add(int x, int y, int z, int color, int fir)
        {
            if (data[x, y, z] == fir)
            {
                int rr = RR.Next(0, 100);
                if (rr < _addChance)
                    data[x, y, z] = color;
            }
        }

        //SPECIAL INTERNAL ROTATION METHOD
        public void RotateRightNoHistory()
        {
            int sX = sizeY;
            int sY = sizeX;

            int[, ,] data2 = new int[sX, sY, sizeZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[sizeY - y - 1, x, z] = data[x, y, z];
                    }
            sizeX = sX;
            sizeY = sY;
            data = data2;
        }

        // CYCLE AND ROTATE        
        public void RotateLeft()
        {
            histEngine.AddEvent("Rotate left", data);

            int sX = sizeY;
            int sY = sizeX;

            int[, ,] data2 = new int[sX, sY, sizeZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[y, sizeX - 1 - x, z] = data[x, y, z];
                    }
            sizeX = sX;
            sizeY = sY;
            data = data2;
        }
        public void RotateRight()
        {
            histEngine.AddEvent("Rotate right", data);

            int sX = sizeY;
            int sY = sizeX;

            int[, ,] data2 = new int[sX, sY, sizeZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[sizeY - y - 1, x, z] = data[x, y, z];
                    }
            sizeX = sX;
            sizeY = sY;
            data = data2;
        }
        public void CycleRight()
        {
            histEngine.AddEvent("Cycle right", data);

            int sX = sizeZ;
            int sZ = sizeX;
            int[, ,] data2 = new int[sX, sizeY, sZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[z, y, sizeX - 1 - x] = data[x, y, z];
                    }
            sizeX = sX;
            sizeZ = sZ;
            int temp = levelTop;
            levelTop = levelSide;
            levelSide = temp;
            data = data2;
        }
        public void CycleLeft()
        {
            histEngine.AddEvent("Cycle left", data);

            int sX = sizeZ;
            int sZ = sizeX;
            int[, ,] data2 = new int[sX, sizeY, sZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[sizeZ - 1 - z, y, x] = data[x, y, z];
                    }
            sizeX = sX;
            sizeZ = sZ;
            int temp = levelTop;
            levelTop = levelSide;
            levelSide = temp;
            data = data2;
        }
        public void CycleFront()
        {
            histEngine.AddEvent("Cycle front", data);

            int sY = sizeZ;
            int sZ = sizeY;
            int[, ,] data2 = new int[sizeX, sY, sZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, z, sizeY - 1 - y] = data[x, y, z];
                    }
            sizeY = sY;
            sizeZ = sZ;
            int temp = levelTop;
            levelTop = levelSide;
            levelSide = temp;
            data = data2;
        }
        public void CycleBack()
        {
            histEngine.AddEvent("Cycle back", data);

            int sY = sizeZ;
            int sZ = sizeY;
            int[, ,] data2 = new int[sizeX, sY, sZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, sizeZ - 1 - z, y] = data[x, y, z];
                    }
            sizeY = sY;
            sizeZ = sZ;
            int temp = levelTop;
            levelTop = levelSide;
            levelSide = temp;
            data = data2;
        }

        //MOVE PIXELS
        public void MoveUp()
        {
            histEngine.AddEvent("Shift up", data);

            int[, ,] data2 = new int[sizeX, sizeY, sizeZ];

            for (int z = 1; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, y, z] = data[x, y, z - 1];
                    }

            if (autoCycle)
            {
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, y, 0] = data[x, y, sizeZ - 1];
                    }
            }

            data = data2;
        }
        public void MoveDown()
        {
            histEngine.AddEvent("Shift down", data);

            int[, ,] data2 = new int[sizeX, sizeY, sizeZ];

            for (int z = 0; z < sizeZ - 1; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, y, z] = data[x, y, z + 1];
                    }
            if (autoCycle)
            {
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, y, sizeZ - 1] = data[x, y, 0];
                    }
            }

            data = data2;
        }
        public void MoveFront()
        {
            histEngine.AddEvent("Move front", data);

            int[, ,] data2 = new int[sizeX, sizeY, sizeZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 1; y < sizeY; y++)
                    {
                        data2[x, y, z] = data[x, y - 1, z];
                    }
            if (autoCycle)
            {
                for (int z = 0; z < sizeZ; z++)
                    for (int x = 0; x < sizeX; x++)
                    {
                        data2[x, 0, z] = data[x, sizeY - 1, z];
                    }
            }

            data = data2;
        }
        public void MoveBack()
        {
            histEngine.AddEvent("Move back", data);

            int[, ,] data2 = new int[sizeX, sizeY, sizeZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY - 1; y++)
                    {
                        data2[x, y, z] = data[x, y + 1, z];
                    }
            if (autoCycle)
            {
                for (int z = 0; z < sizeZ; z++)
                    for (int x = 0; x < sizeX; x++)
                    {
                        data2[x, sizeY - 1, z] = data[x, 0, z];
                    }
            }

            data = data2;
        }
        public void MoveRight()
        {
            histEngine.AddEvent("Move right", data);

            int[, ,] data2 = new int[sizeX, sizeY, sizeZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 1; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, y, z] = data[x - 1, y, z];
                    }
            if (autoCycle)
            {
                for (int z = 0; z < sizeZ; z++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[0, y, z] = data[sizeX - 1, y, z];
                    }
            }
            data = data2;
        }
        public void MoveLeft()
        {
            histEngine.AddEvent("Move left", data);

            int[, ,] data2 = new int[sizeX, sizeY, sizeZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX - 1; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, y, z] = data[x + 1, y, z];
                    }
            if (autoCycle)
            {
                for (int z = 0; z < sizeZ; z++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[sizeX - 1, y, z] = data[0, y, z];
                    }
            }
            data = data2;
        }

        // MIRROR ALL PIXELS        
        public void MirrorX()
        {
            histEngine.AddEvent("Mirror X", data);

            int[, ,] data2 = new int[sizeX, sizeY, sizeZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, y, z] = data[sizeX - 1 - x, y, z];
                    }
            data = data2;
        }
        public void MirrorY()
        {
            histEngine.AddEvent("Mirror Y", data);

            int[, ,] data2 = new int[sizeX, sizeY, sizeZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, y, z] = data[x, sizeY - 1 - y, z];
                    }
            data = data2;
        }
        public void MirrorZ()
        {
            histEngine.AddEvent("Mirror Z", data);

            int[, ,] data2 = new int[sizeX, sizeY, sizeZ];

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        data2[x, y, z] = data[x, y, sizeZ - 1 - z];
                    }
            data = data2;
        }

        //HISTORY PROCESSING
        public void Undo()
        {
            TIconHistory TT = histEngine.UndoEvent();
            if (TT == null) return;
            data = TT.data;
        }
        public void Redo()
        {
            TIconHistory TT = histEngine.RedoEvent();
            if (TT == null) return;
            data = TT.data;
        }
        public void UndoAll()
        {
            TIconHistory TT = histEngine.UndoAllEvents();
            if (TT == null) return;
            data = TT.data;
        }
        public void RedoAll()
        {
            TIconHistory TT = histEngine.RedoEventAll();
            if (TT == null) return;
            data = TT.data;
        }
        public void ClearHistory()
        {
            histEngine.ClearHistory();
        }

#endregion

#region user interfejs

        // CLICKING ON TOP AND SIDE PANEL
        public int ClickTop(MouseEventArgs e, int color, bool first)
        {
            int xx = (e.X - 32) * sizeX / panelDimensionTopX;
            int yy = e.Y * sizeY / panelDimensionTopY;
            if (xx >= sizeX) return color;
            if (yy >= sizeY) return color;
            if (xx < 0) return color;
            if (yy < 0) return color;

            if (lastMouseTopX == xx & lastMouseTopY == yy)
            {
                lastMouseTopX = xx;
                lastMouseTopY = yy;
                return 100000;
            }

            lastMouseTopX = xx;
            lastMouseTopY = yy;

            //RIGHT RMB SO FAR COLOR PICKER
            if (e.Button == MouseButtons.Right)
            {
                int dd = data[xx, yy, levelTop];
                if (dd < 0) return -dd;
                return dd;
            }

            int colMod = 1;
            if (alphaMode) colMod = -1;

            //LMB so far only drawing
            if (e.Button == MouseButtons.Left)
            {
                //SINGLE PIXEL
                if (drawType == 0)
                {
                    if(first)
                        histEngine.AddEvent("Draw single pixel", data);

                    data[xx, yy, levelTop] = color * colMod;

                    if (deepMode)
                    {
                        for (int i = 0; i < deepLevel; i++)
                        {
                            if (levelTop + i < sizeZ)
                                data[xx, yy, levelTop + i] = color * colMod;
                        }
                    }
                }

                //LARGE PIXEL BRUSH
                if (drawType == 1)
                {
                    if (first)
                        histEngine.AddEvent("Draw brush 3x3", data);

                    for (int x = -1; x <= 1; x++)
                        for (int y = -1; y <= 1; y++)
                        {
                            if (xx + x < 0) continue;
                            if (yy + y < 0) continue;
                            if (xx + x >= sizeX) continue;
                            if (yy + y >= sizeY) continue;

                            if (addType)
                                Add(x + xx, y + yy, levelTop, color * colMod, 0);
                            if (randomType)
                                Noise(x + xx, y + yy, levelTop, color * colMod);
                            if (removeType)
                                Remove(x + xx, y + yy, levelTop, color * colMod);

                            if (!randomType & !removeType & !addType)
                                data[xx + x, yy + y, levelTop] = color * colMod;

                            if (deepMode)
                            {
                                for (int i = 0; i < deepLevel; i++)
                                {
                                    if (levelTop + i < sizeZ)
                                    {
                                        if (addType)
                                            Add(x + xx, y + yy, levelTop + i, color * colMod, 0);
                                        if (randomType)
                                            Noise(x + xx, y + yy, levelTop + i, color * colMod);
                                        if (removeType)
                                            Remove(x + xx, y + yy, levelTop + i, color * colMod);

                                        if (!randomType & !removeType & !addType)
                                            data[xx + x, yy + y, levelTop + i] = color * colMod;

                                    }
                                }
                            }
                        }
                }

                //LAYER
                if (drawType == 2)
                {
                    if (first)
                        histEngine.AddEvent("Draw whole layer", data);

                    int fir = data[xx, yy, levelTop];
                    for (int x = 0; x < sizeX; x++)
                        for (int y = 0; y < sizeY; y++)
                            if (data[x, y, levelTop] == fir)
                            {
                                if (addType)
                                    Add(x, y, levelTop, color * colMod, fir * colMod);
                                if (randomType)
                                    Noise(x, y, levelTop, color * colMod);
                                if (removeType)
                                    Remove(x, y, levelTop, color * colMod);

                                if (!randomType & !removeType & !addType)
                                    data[x, y, levelTop] = color * colMod;


                                if (deepMode)
                                {
                                    for (int i = 0; i < deepLevel; i++)
                                    {
                                        if (levelTop + i < sizeZ)
                                        {
                                            if (addType)
                                                Add(x, y, levelTop + i, color * colMod, 0);
                                            if (randomType)
                                                Noise(x, y, levelTop + i, color * colMod);
                                            if (removeType)
                                                Remove(x, y, levelTop + i, color * colMod);

                                            if (!randomType & !removeType & !addType)
                                                data[x, y, levelTop + i] = color * colMod;

                                        }
                                    }
                                }
                            }

                }

                //WHOLE ICON
                if (drawType == 4)
                {
                    if (first)
                        histEngine.AddEvent("Replace all pixels", data);

                    int fir = data[xx, yy, levelTop];
                    for (int z = 0; z < sizeZ; z++)
                        for (int x = 0; x < sizeX; x++)
                            for (int y = 0; y < sizeY; y++)
                                if (data[x, y, z] == fir)
                                {
                                    if (addType)
                                        Add(x, y, z, color * colMod, fir * colMod);
                                    if (randomType)
                                        Noise(x, y, z, color * colMod);
                                    if (removeType)
                                        Remove(x, y, z, color * colMod);

                                    if (!randomType & !removeType & !addType)
                                        data[x, y, z] = color * colMod;
                                }

                }
            }

            return color;
        }
        public int ClickSide(MouseEventArgs e, int color, bool first)
        {
            int xx = (e.X - 32) * sizeX / panelDimensionSideX;
            int zz = sizeZ - 1 - e.Y * sizeZ / panelDimensionSideY;
            if (xx >= sizeX) return color;
            if (zz >= sizeZ) return color;
            if (xx < 0) return color;
            if (zz < 0) return color;

            if (lastMouseSideX == xx & lastMouseSideY == zz)
            {
                lastMouseSideX = xx;
                lastMouseSideY = zz;
                return 100000;
            }

            lastMouseSideX = xx;
            lastMouseSideY = zz;

            //RMB SO FAR COLOR PICKER
            if (e.Button == MouseButtons.Right)
            {
                int dd = data[xx, levelSide, zz];
                if (dd < 0) return -dd;
                return dd;
            }

            int colMod = 1;
            if (alphaMode) colMod = -1;

            //LMB SO FAR DRAWING
            if (e.Button == MouseButtons.Left)
            {
                //SINGLE PIXEL
                if (drawType == 0)
                {
                    if (first)
                        histEngine.AddEvent("Draw single pixel", data);

                    data[xx, levelSide, zz] = color * colMod;

                    if (deepMode)
                    {
                        for (int i = 0; i < deepLevel; i++)
                        {
                            if (levelSide + i < sizeY)
                                data[xx, levelSide + i, zz] = color * colMod;
                        }
                    }
                }

                //LARGE PIXEL BRUSH
                if (drawType == 1)
                {
                    if (first)
                        histEngine.AddEvent("Draw brush 3x3 pixels", data);

                    for (int x = -1; x <= 1; x++)
                        for (int z = -1; z <= 1; z++)
                        {
                            if (xx + x < 0) continue;
                            if (zz + z < 0) continue;
                            if (xx + x >= sizeX) continue;
                            if (zz + z >= sizeZ) continue;

                            if (addType)
                                Add(x + xx, levelSide, zz + z, color * colMod, 0);
                            if (randomType)
                                Noise(x + xx, levelSide, zz + z, color * colMod);
                            if (removeType)
                                Remove(x + xx, levelSide, zz + z, color * colMod);

                            if (!randomType & !removeType & !addType)
                                data[xx + x, levelSide, zz + z] = color * colMod;

                            if (deepMode)
                            {
                                for (int i = 0; i < deepLevel; i++)
                                {
                                    if (levelSide + i < sizeY)
                                    {
                                        if (addType)
                                            Add(x + xx, levelSide + i, z + zz, color * colMod, 0);
                                        if (randomType)
                                            Noise(x + xx, levelSide + i, z + zz, color * colMod);
                                        if (removeType)
                                            Remove(x + xx, levelSide + i, z + zz, color * colMod);

                                        if (!randomType & !removeType & !addType)
                                            data[x + xx, levelSide + i, z + zz] = color * colMod;

                                    }
                                }
                            }
                        }
                }

                //LAYER
                if (drawType == 2)
                {
                    if (first)
                        histEngine.AddEvent("Draw single layer", data);

                    int fir = data[xx, levelSide, zz];
                    for (int x = 0; x < sizeX; x++)
                        for (int z = 0; z < sizeZ; z++)
                            if (data[x, levelSide, z] == fir)
                            {
                                if (addType)
                                    Add(x, levelSide, z, color * colMod, fir * colMod);
                                if (randomType)
                                    Noise(x, levelSide, z, color * colMod);
                                if (removeType)
                                    Remove(x, levelSide, z, color * colMod);

                                if (!randomType & !removeType & !addType)
                                    data[x, levelSide, z] = color * colMod;

                                if (deepMode)
                                {
                                    for (int i = 0; i < deepLevel; i++)
                                    {
                                        if (levelSide + i < sizeY)
                                        {
                                            if (addType)
                                                Add(x, levelSide + i, z, color * colMod, 0);
                                            if (randomType)
                                                Noise(x, levelSide + i, z, color * colMod);
                                            if (removeType)
                                                Remove(x, levelSide + i, z, color * colMod);

                                            if (!randomType & !removeType & !addType)
                                                data[x, levelSide + i, z] = color * colMod;

                                        }
                                    }
                                }
                            }

                }

                //WHOLE ICON
                if (drawType == 4)
                {
                    if (first)
                        histEngine.AddEvent("Replace all pixels", data);

                    int fir = data[xx, levelSide, zz];
                    for (int z = 0; z < sizeZ; z++)
                        for (int x = 0; x < sizeX; x++)
                            for (int y = 0; y < sizeY; y++)
                                if (data[x, y, z] == fir)
                                {
                                    if (addType)
                                        Add(x, y, z, color * colMod, fir * colMod);
                                    if (randomType)
                                        Noise(x, y, z, color * colMod);
                                    if (removeType)
                                        Remove(x, y, z, color * colMod);

                                    if (!randomType & !removeType & !addType)
                                        data[x, y, z] = color * colMod;
                                }

                }
            }
            return color;
        }

        // CHANGE ZOOM
        public void ChangeZoom(int dd)
        {
            if (dd > 0) zoomLevel++;
            if (dd < 0) zoomLevel--;
            if (dd == 0) zoomLevel = zoomLevelBegin;
            if (zoomLevel >= zoomLevelValueMax) zoomLevel = zoomLevelValueMax;
            if (zoomLevel < 0) zoomLevel = 0;
        }

        // CHANGE OUTPUT OFFSET WHEN PRESSING RIGHT MOUSE BUTTON ON OUPUT
        internal void ChangeOffset(MouseEventArgs e, Size size)
        {
            int xx = size.Width / 2 - e.X;
            int yy = size.Height / 2 - e.Y;

            outPutOffsetX += xx / 3;
            outPutOffsetY += yy / 3;
        }

#endregion

#region drawing

        // DRAW OUTPUT ICON ON THE SCREEN
        public void DrawOnScreen(Graphics G)
        {
            //draw one particular imge on screen
            DrawAllIntoMemory();

            G.SmoothingMode = SmoothingMode.HighSpeed;
            G.CompositingQuality = CompositingQuality.HighSpeed;
            G.InterpolationMode = InterpolationMode.NearestNeighbor;
            G.DrawImage(bmpOut, outPutOffsetX, outPutOffsetY, bmpX * GetZoomLevel, bmpY * GetZoomLevel);
        }

        // DRAW ALL OUTPUT ICONS INTO MEMORY
        public Bitmap DrawAllIntoMemory()
        {
            //draw from iso
            if (iconRotate % 2 == 0)
            {
                if (iconRotate == 0)
                    bmpOut = DrawSingleMemoryIso();
                RotateRightNoHistory();

                if (iconRotate == 2)
                    bmpOut = DrawSingleMemoryIso();
                RotateRightNoHistory();

                if (iconRotate == 4)
                    bmpOut = DrawSingleMemoryIso();
                RotateRightNoHistory();

                if (iconRotate == 6)
                    bmpOut = DrawSingleMemoryIso();
                RotateRightNoHistory();
            }

            //draw from normal
            if (iconRotate % 2 == 1)
            {
                if (iconRotate == 1)
                    bmpOut = DrawSingleMemoryTop();
                RotateRightNoHistory();

                if (iconRotate == 3)
                    bmpOut = DrawSingleMemoryTop();
                RotateRightNoHistory();

                if (iconRotate == 5)
                    bmpOut = DrawSingleMemoryTop();
                RotateRightNoHistory();

                if (iconRotate == 7)
                    bmpOut = DrawSingleMemoryTop();
                RotateRightNoHistory();
            }
            return bmpOut;
        }

        //DRAW SINGLE OUTPUT ICON INTO MEMORY THIS IS FOR ISO VIEW
        public Bitmap DrawSingleMemoryIso()
        {
            int rr = GetZoomLevel;

            Bitmap B = new Bitmap(bmpX * rr, bmpY * rr);
            Graphics G2 = Graphics.FromImage(B);

            G2.SmoothingMode = SmoothingMode.HighSpeed;
            G2.CompositingQuality = CompositingQuality.HighSpeed;
            G2.InterpolationMode = InterpolationMode.NearestNeighbor;

            G2.FillRectangle(backGroundSolid, 0, 0, bmpX * rr, bmpY * rr);
            Pen outline = new Pen(Color.FromArgb(_borderAlpha * 255 / 100, 0, 0, 0), rr * 1.0f * _borderSize / 10);
            Pen glow = new Pen(Color.FromArgb(_glowAlpha * 255 / 100, 255, 255, 255), rr * 1.0f * _glowSize / 10);
            outline.StartCap = LineCap.Round;
            glow.StartCap = LineCap.Round;
            outline.EndCap = LineCap.Round;
            glow.EndCap = LineCap.Round;

            SolidBrush markerBrush = new SolidBrush(_markerColor.ToColor());
            SolidBrush ppp1 = new SolidBrush(Color.Transparent);
            SolidBrush ppp2 = new SolidBrush(Color.Transparent);
            SolidBrush ppp3 = new SolidBrush(Color.Transparent);

            //light map 
            int[, ,] shadowMap = new int[sizeX, sizeY, sizeZ];
            if (upperShadow)
            {
                for (int y = 0; y < sizeY; y++)
                    for (int x = 0; x < sizeX; x++)
                    {
                        int shad = sizeZ * _sizeForShadow / 100 + 1;
                        for (int z = sizeZ - 1; z >= 0; z--)
                        {
                            if (data[x, y, z] > 0) shad--;
                            shadowMap[x, y, z] = shad;
                        }
                    }
            }

            int fromZ = 0;
            int intoZ = sizeZ;

            //LIMIT VIEW TO ONION
            if (onionLevel == 1)
                intoZ = levelTop + 1;
            if (onionLevel == 2)
            {
                fromZ = levelTop;
                intoZ = levelTop + 1;
            }

            //points array
            Point[] sss = new Point[4];

            for (int z = fromZ; z < intoZ; z++)
            {
                //DRAW MARKER
                if (drawMarkersOnScreen)
                    if (z == levelTop)
                        G2.FillRectangle(markerBrush, 0, 0, bmpX * rr, bmpY * rr);

                for (int y = 0; y < sizeY; y++)
                    for (int x = sizeX - 1; x >= 0; x--)
                    {
                        int S = data[x, y, z];
                        int sad = shadowMap[x, y, z];

                        int xx2 = (y + x) * 2 * rr;
                        int yy2 = ((y - x) - z * 2 + bmpY - sizeY - 2) * rr;

                        //NORMAL MODE
                        if (S > 0)
                        {
                            ppp1.Color = colors1[S].ToColor();
                            ppp2.Color = colors2[S].ToColor();
                            ppp3.Color = colors3[S].ToColor();

                            if (sad < sizeZ * _sizeForShadow / 100)
                                if (upperShadow)
                                {
                                    ppp1.Color = LightDownColor(ppp1.Color, sad);
                                    ppp2.Color = LightDownColor(ppp2.Color, sad);
                                    ppp3.Color = LightDownColor(ppp3.Color, sad);
                                }

                            int sOther = 0;

                            sOther = GetPixelCell2(data, x, y, z + 1);
                            if (sOther <= 0)
                            {
                                sss[0].X = xx2;
                                sss[0].Y = yy2;
                                sss[1].X = xx2 + 2 * rr;
                                sss[1].Y = yy2 + 1 * rr;
                                sss[2].X = xx2 + 4 * rr;
                                sss[2].Y = yy2;
                                sss[3].X = xx2 + 2 * rr;
                                sss[3].Y = yy2 - 1 * rr;                                
                                G2.FillPolygon(ppp1, sss);
                            }

                            sOther = GetPixelCell2(data, x - 1, y, z);
                            if (sOther <= 0)
                            {
                                sss[0].X = xx2;
                                sss[0].Y = yy2;
                                sss[1].X = xx2 + 2 * rr;
                                sss[1].Y = yy2 + 1 * rr;
                                sss[2].X = xx2 + 2 * rr;
                                sss[2].Y = yy2 + 3 * rr;
                                sss[3].X = xx2;
                                sss[3].Y = yy2 + 2 * rr;                                   
                                G2.FillPolygon(ppp2, sss);
                            }

                            sOther = GetPixelCell2(data, x, y + 1, z);
                            if (sOther <= 0)
                            {
                                sss[0].X = xx2 + 2 * rr;
                                sss[0].Y = yy2 + 1 * rr;
                                sss[1].X = xx2 + 4 * rr;
                                sss[1].Y = yy2;
                                sss[2].X = xx2 + 4 * rr;
                                sss[2].Y = yy2 + 2 * rr;
                                sss[3].X = xx2 + 2 * rr;
                                sss[3].Y = yy2 + 3 * rr;                                   
                                G2.FillPolygon(ppp3, sss);
                            }


                            if (enveloperMode)
                            {
                                sOther = GetPixelCell2(data, x, y, z - 1);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(outline, xx2, yy2 + 2 * rr, xx2 + 2 * rr, yy2 + 3 * rr);
                                    G2.DrawLine(outline, xx2 + 4 * rr, yy2 + 2 * rr, xx2 + 2 * rr, yy2 + 3 * rr);
                                }

                                sOther = GetPixelCell2(data, x, y - 1, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(outline, xx2, yy2, xx2 + 2 * rr, yy2 - 1 * rr);
                                    G2.DrawLine(outline, xx2, yy2, xx2, yy2 + 2 * rr);
                                }

                                sOther = GetPixelCell2(data, x + 1, y, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(outline, xx2 + 4 * rr, yy2, xx2 + 4 * rr, yy2 + 2 * rr);
                                    G2.DrawLine(outline, xx2 + 4 * rr, yy2, xx2 + 2 * rr, yy2 - 1 * rr);
                                }
                            }


                            if (glowMode)
                            {
                                sOther = GetPixelCell2(data, x, y, z - 1);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2, yy2 + 2 * rr, xx2 + 2 * rr, yy2 + 3 * rr);
                                    G2.DrawLine(glow, xx2 + 4 * rr, yy2 + 2 * rr, xx2 + 2 * rr, yy2 + 3 * rr);
                                }

                                sOther = GetPixelCell2(data, x, y - 1, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2, yy2, xx2 + 2 * rr, yy2 - 1 * rr);
                                    G2.DrawLine(glow, xx2, yy2, xx2, yy2 + 2 * rr);
                                }

                                sOther = GetPixelCell2(data, x + 1, y, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2 + 4 * rr, yy2, xx2 + 4 * rr, yy2 + 2 * rr);
                                    G2.DrawLine(glow, xx2 + 4 * rr, yy2, xx2 + 2 * rr, yy2 - 1 * rr);
                                }
                            }
                        }

                        //ALPHA MODE
                        if (S < 0)
                        {
                            ppp1.Color = Color.FromArgb(_alpha * 255 / 100, colors1[-S].R, colors1[-S].G, colors1[-S].B);
                            ppp2.Color = Color.FromArgb(_alpha * 255 / 100, colors2[-S].R, colors2[-S].G, colors2[-S].B);
                            ppp3.Color = Color.FromArgb(_alpha * 255 / 100, colors3[-S].R, colors3[-S].G, colors3[-S].B);

                            int sOther = 0;

                            sOther = GetPixelCell2(data, x, y, z + 1);
                            if (sOther == 0)
                            {
                                sss[0].X = xx2;
                                sss[0].Y = yy2;
                                sss[1].X = xx2 + 2 * rr;
                                sss[1].Y = yy2 + 1 * rr;
                                sss[2].X = xx2 + 4 * rr;
                                sss[2].Y = yy2;
                                sss[3].X = xx2 + 2 * rr;
                                sss[3].Y = yy2 - 1 * rr; 
                                G2.FillPolygon(ppp1, sss);
                            }

                            sOther = GetPixelCell2(data, x - 1, y, z);
                            if (sOther == 0)
                            {
                                sss[0].X = xx2;
                                sss[0].Y = yy2;
                                sss[1].X = xx2 + 2 * rr;
                                sss[1].Y = yy2 + 1 * rr;
                                sss[2].X = xx2 + 2 * rr;
                                sss[2].Y = yy2 + 3 * rr;
                                sss[3].X = xx2;
                                sss[3].Y = yy2 + 2 * rr;
                                G2.FillPolygon(ppp2, sss);
                            }

                            sOther = GetPixelCell2(data, x, y + 1, z);
                            if (sOther == 0)
                            {
                                sss[0].X = xx2 + 2 * rr;
                                sss[0].Y = yy2 + 1 * rr;
                                sss[1].X = xx2 + 4 * rr;
                                sss[1].Y = yy2;
                                sss[2].X = xx2 + 4 * rr;
                                sss[2].Y = yy2 + 2 * rr;
                                sss[3].X = xx2 + 2 * rr;
                                sss[3].Y = yy2 + 3 * rr;
                                G2.FillPolygon(ppp3, sss);
                            }

                            if (glowMode)
                            {
                                sOther = GetPixelCell2(data, x, y, z - 1);
                                if (sOther == 0)
                                {
                                    G2.DrawLine(glow, xx2, yy2 + 2 * rr, xx2 + 2 * rr, yy2 + 3 * rr);
                                    G2.DrawLine(glow, xx2 + 4 * rr, yy2 + 2 * rr, xx2 + 2 * rr, yy2 + 3 * rr);
                                }

                                sOther = GetPixelCell2(data, x, y - 1, z);
                                if (sOther == 0)
                                {
                                    G2.DrawLine(glow, xx2, yy2, xx2 + 2 * rr, yy2 - 1 * rr);
                                    G2.DrawLine(glow, xx2, yy2, xx2, yy2 + 2 * rr);
                                }

                                sOther = GetPixelCell2(data, x + 1, y, z);
                                if (sOther == 0)
                                {
                                    G2.DrawLine(glow, xx2 + 4 * rr, yy2, xx2 + 4 * rr, yy2 + 2 * rr);
                                    G2.DrawLine(glow, xx2 + 4 * rr, yy2, xx2 + 2 * rr, yy2 - 1 * rr);
                                }
                            }

                        }

                    }
            }
            G2.Dispose();

            return B;
        }

        //DRAW SINGLE OUTPUT ICON INTO MEMORY THIS IS TOP VIEW
        public Bitmap DrawSingleMemoryTop()
        {
            int rr = GetZoomLevel;

            Bitmap B = new Bitmap(bmpX * rr, bmpY * rr);
            Graphics G2 = Graphics.FromImage(B);

            G2.SmoothingMode = SmoothingMode.HighSpeed;
            G2.CompositingQuality = CompositingQuality.HighSpeed;
            G2.InterpolationMode = InterpolationMode.NearestNeighbor;

            SolidBrush ppp1 = new SolidBrush(Color.Black);
            SolidBrush ppp2 = new SolidBrush(Color.Black);
            SolidBrush ppp3 = new SolidBrush(Color.Black);

            SolidBrush markerBrush = new SolidBrush(_markerColor.ToColor());
            G2.FillRectangle(backGroundSolid, 0, 0, bmpX * rr, bmpY * rr);
            Pen outline = new Pen(Color.FromArgb(_borderAlpha * 255 / 100, 0, 0, 0), rr * 1.0f * _borderSize / 10);
            Pen glow = new Pen(Color.FromArgb(_glowAlpha * 255 / 100, 255, 255, 255), rr * 1.0f * _glowSize / 10);
            outline.StartCap = LineCap.Round;
            glow.StartCap = LineCap.Round;
            outline.EndCap = LineCap.Round;
            glow.EndCap = LineCap.Round;

            //light map 
            int[, ,] shadowMap = new int[sizeX, sizeY, sizeZ];
            if (upperShadow)
            {
                for (int y = 0; y < sizeY; y++)
                    for (int x = 0; x < sizeX; x++)
                    {
                        int shad = sizeZ * _sizeForShadow / 100 + 1;
                        for (int z = sizeZ - 1; z >= 0; z--)
                        {
                            if (data[x, y, z] > 0) shad--;
                            shadowMap[x, y, z] = shad;
                        }
                    }
            }

            int fromZ = 0;
            int intoZ = sizeZ;

            if (onionLevel == 1)
            {
                intoZ = levelTop + 1;
            }

            if (onionLevel == 2)
            {
                fromZ = levelTop;
                intoZ = levelTop + 1;
            }

            for (int z = fromZ; z < intoZ; z++)
            {
                if (drawMarkersOnScreen)
                    if (z == levelTop)
                        G2.FillRectangle(markerBrush, 0, 0, bmpX * rr, bmpY * rr);

                for (int y = 0; y < sizeY; y++)
                    for (int x = sizeX - 1; x >= 0; x--)
                    {
                        int S = data[x, y, z];
                        int sad = shadowMap[x, y, z];

                        int xx2 = x * 3 * rr;
                        int yy2 = ((y - 1) * 2 - z * 2 + sizeY * 2) * rr;

                        //NORMAL MODE
                        if (S > 0)
                        {
                            ppp1.Color = colors1[S].ToColor();
                            ppp2.Color = colors2[S].ToColor();
                            ppp3.Color = colors3[S].ToColor();

                            if (sad < sizeZ * _sizeForShadow / 100)
                                if (upperShadow)
                                {
                                    ppp1.Color = LightDownColor(ppp1.Color, sad);
                                    ppp2.Color = LightDownColor(ppp2.Color, sad);
                                    ppp3.Color = LightDownColor(ppp3.Color, sad);
                                }

                            int sOther = 0;

                            sOther = GetPixelCell2(data, x, y, z + 1);
                            if (sOther <= 0)
                            {
                                Point[] sss = { new Point(xx2, yy2), new Point(xx2 + 3 * rr, yy2), 
                                                  new Point(xx2 + 3 * rr, yy2 + 2 * rr), new Point(xx2, yy2 + 2 * rr) };
                                G2.FillPolygon(ppp1, sss);
                            }

                            sOther = GetPixelCell2(data, x, y + 1, z);
                            if (sOther <= 0)
                            {
                                Point[] sss3 = { new Point(xx2, yy2 + 2 * rr), new Point(xx2 + 3 * rr, yy2 + 2 * rr), 
                                                   new Point(xx2 + 3 * rr, yy2 + 4 * rr), new Point(xx2, yy2 + 4 * rr) };
                                G2.FillPolygon(ppp3, sss3);
                            }


                            if (enveloperMode)
                            {
                                sOther = GetPixelCell2(data, x, y - 1, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(outline, xx2, yy2, xx2 + 3 * rr, yy2);
                                }

                                sOther = GetPixelCell2(data, x, y, z - 1);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(outline, xx2, yy2 + 4 * rr, xx2 + 3 * rr, yy2 + 4 * rr);
                                }

                                sOther = GetPixelCell2(data, x - 1, y, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(outline, xx2, yy2, xx2, yy2 + 4 * rr);
                                }

                                sOther = GetPixelCell2(data, x + 1, y, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(outline, xx2 + 3 * rr, yy2, xx2 + 3 * rr, yy2 + 4 * rr);
                                }
                            }


                            if (glowMode)
                            {
                                sOther = GetPixelCell2(data, x, y - 1, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2, yy2, xx2 + 3 * rr, yy2);
                                }

                                sOther = GetPixelCell2(data, x, y, z - 1);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2, yy2 + 4 * rr, xx2 + 3 * rr, yy2 + 4 * rr);
                                }

                                sOther = GetPixelCell2(data, x - 1, y, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2, yy2, xx2, yy2 + 4 * rr);
                                }

                                sOther = GetPixelCell2(data, x + 1, y, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2 + 3 * rr, yy2, xx2 + 3 * rr, yy2 + 4 * rr);
                                }
                            }
                        }

                        //ALPHA MODE
                        if (S < 0)
                        {
                            ppp1.Color = Color.FromArgb(_alpha * 255 / 100, colors1[-S].R, colors1[-S].G, colors1[-S].B);
                            ppp2.Color = Color.FromArgb(_alpha * 255 / 100, colors2[-S].R, colors2[-S].G, colors2[-S].B);
                            ppp3.Color = Color.FromArgb(_alpha * 255 / 100, colors3[-S].R, colors3[-S].G, colors3[-S].B);

                            int sOther = 0;

                            sOther = GetPixelCell2(data, x, y, z + 1);
                            if (sOther <= 0)
                            {
                                Point[] sss = { new Point(xx2, yy2), new Point(xx2 + 3 * rr, yy2), 
                                                  new Point(xx2 + 3 * rr, yy2 + 2 * rr), new Point(xx2, yy2 + 2 * rr) };
                                G2.FillPolygon(ppp1, sss);
                            }

                            sOther = GetPixelCell2(data, x, y + 1, z);
                            if (sOther <= 0)
                            {
                                Point[] sss3 = { new Point(xx2, yy2 + 2 * rr), new Point(xx2 + 3 * rr, yy2 + 2 * rr), 
                                                   new Point(xx2 + 3 * rr, yy2 + 4 * rr), new Point(xx2, yy2 + 4 * rr) };
                                G2.FillPolygon(ppp3, sss3);
                            }

                            if (glowMode)
                            {
                                sOther = GetPixelCell2(data, x, y - 1, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2, yy2, xx2 + 3 * rr, yy2);
                                }

                                sOther = GetPixelCell2(data, x, y, z - 1);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2, yy2 + 4 * rr, xx2 + 3 * rr, yy2 + 4 * rr);
                                }

                                sOther = GetPixelCell2(data, x - 1, y, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2, yy2, xx2, yy2 + 4 * rr);
                                }

                                sOther = GetPixelCell2(data, x + 1, y, z);
                                if (sOther <= 0)
                                {
                                    G2.DrawLine(glow, xx2 + 3 * rr, yy2, xx2 + 3 * rr, yy2 + 4 * rr);
                                }
                            }

                        }
                    }
            }
            G2.Dispose();
            return B;
        }

        // DRAW ALL INTO MEMORY AND SAVE INTO FILE
        public void DrawAllMemoryAndSaveToFile(string where)
        {
            Bitmap[] BB = new Bitmap[8];

            BB[0] = DrawSingleMemoryIso();
            BB[1] = DrawSingleMemoryTop();
            RotateRightNoHistory();

            BB[2] = DrawSingleMemoryIso();
            BB[3] = DrawSingleMemoryTop();
            RotateRightNoHistory();

            BB[4] = DrawSingleMemoryIso();
            BB[5] = DrawSingleMemoryTop();
            RotateRightNoHistory();

            BB[6] = DrawSingleMemoryIso();
            BB[7] = DrawSingleMemoryTop();
            RotateRightNoHistory();

            SaveToPng(BB, where);
        }

        // DRAW ALL INTO MEMORY AND SAVE INTO FILE
        public void DrawAllSaveIntoFiles(string where, bool alias, bool singleFile, int[] views)
        {
            Bitmap[] BB = new Bitmap[8];

            if (views[0] > 0) BB[0] = DrawSingleMemoryIso();
            if (views[1] > 0) BB[1] = DrawSingleMemoryTop();
            RotateRightNoHistory();

            if (views[2] > 0) BB[2] = DrawSingleMemoryIso();
            if (views[3] > 0) BB[3] = DrawSingleMemoryTop();
            RotateRightNoHistory();

            if (views[4] > 0) BB[4] = DrawSingleMemoryIso();
            if (views[5] > 0) BB[5] = DrawSingleMemoryTop();
            RotateRightNoHistory();

            if (views[6] > 0) BB[6] = DrawSingleMemoryIso();
            if (views[7] > 0) BB[7] = DrawSingleMemoryTop();
            RotateRightNoHistory();

            //SINGLE FILE
            if (singleFile)
                SaveToPng(BB, where, views);
            else
            {
                string www = where.Substring(0, where.Length - 4);

                for (int i = 0; i < 8; i++)
                    if (views[i] > 0) SaveToPng(BB[i], www + "00" + i + ".png");
            }
        }

        // EXPORT TO PNG
        public void SaveToPng(Bitmap[] PP, string where)
        {
            int rr = GetZoomLevel;

            Bitmap BB = new Bitmap(bmpX * 8 * rr * _outPutSize / 100, bmpY * rr * _outPutSize / 100);
            Graphics GG = Graphics.FromImage(BB);

            GG.SmoothingMode = SmoothingMode.AntiAlias;
            GG.CompositingQuality = CompositingQuality.HighQuality;
            GG.InterpolationMode = InterpolationMode.HighQualityBicubic;

            for (int i = 0; i < 8; i++)
                GG.DrawImage(PP[i], bmpX * i * rr * _outPutSize / 100, 0, bmpX * rr * _outPutSize / 100, bmpY * rr * _outPutSize / 100);

            BB.Save(where);
            BB.Dispose();
        }
        public void SaveToPng(Bitmap[] PP, string where, int[] cel)
        {
            int rr = GetZoomLevel;

            Bitmap BB = new Bitmap(bmpX * 8 * rr * _outPutSize / 100, bmpY * rr * _outPutSize / 100);
            Graphics GG = Graphics.FromImage(BB);

            GG.SmoothingMode = SmoothingMode.AntiAlias;
            GG.CompositingQuality = CompositingQuality.HighQuality;
            GG.InterpolationMode = InterpolationMode.HighQualityBicubic;

            for (int i = 0; i < 8; i++)
                if (cel[i] > 0)
                    GG.DrawImage(PP[i], bmpX * i * rr * _outPutSize / 100, 0, bmpX * rr * _outPutSize / 100, bmpY * rr * _outPutSize / 100);

            BB.Save(where);
            BB.Dispose();
        }
        public void SaveToPng(Bitmap PP, string where)
        {
            int rr = GetZoomLevel;

            Bitmap BB = new Bitmap(bmpX * rr * _outPutSize / 100, bmpY * rr * _outPutSize / 100);
            Graphics GG = Graphics.FromImage(BB);

            GG.SmoothingMode = SmoothingMode.AntiAlias;
            GG.CompositingQuality = CompositingQuality.HighQuality;
            GG.InterpolationMode = InterpolationMode.HighQualityBicubic;

            GG.DrawImage(PP, 0, 0, bmpX * rr * _outPutSize / 100, bmpY * rr * _outPutSize / 100);

            BB.Save(where);
            BB.Dispose();
        }

        // DRAW COLOR PANEL
        public void DrawColorsOnPanel(Graphics G, int selectedColor)
        {
            G.SmoothingMode = SmoothingMode.HighSpeed;
            G.CompositingQuality = CompositingQuality.HighSpeed;
            G.InterpolationMode = InterpolationMode.NearestNeighbor;

            SolidBrush SS = new SolidBrush(Color.Black);

            for (int i = 0; i < 256; i++)
            {
                SS.Color = colorsBase[i].ToColor();
                int x = i % palleteXLenght;
                int y = i / palleteXLenght;
                G.FillRectangle(SS, x * palleteX, y * palleteY, palleteX, palleteY);
            }

            G.DrawRectangle(Pens.White, (selectedColor % palleteXLenght) * palleteX + 1,
                (selectedColor / palleteXLenght) * palleteY + 1, palleteX - 2, palleteY - 2);
            G.DrawRectangle(Pens.Black, (selectedColor2 % palleteXLenght) * palleteX + 1,
                (selectedColor2 / palleteXLenght) * palleteY + 1, palleteX - 2, palleteY - 2);
        }

        // DRAW SIDE AND TOP PANEL EDITOR
        public void DrawTop(Graphics G)
        {
            G.SmoothingMode = SmoothingMode.HighSpeed;
            G.CompositingQuality = CompositingQuality.HighSpeed;
            G.InterpolationMode = InterpolationMode.NearestNeighbor;

            SolidBrush SS = new SolidBrush(Color.Black);
            SolidBrush KK = new SolidBrush(Color.FromArgb(180, 0, 0, 0));
            SolidBrush KK2 = new SolidBrush(Color.FromArgb(90, 0, 0, 0));

            float cellX = panelDimensionTopX * 1.0f / sizeX;
            float cellY = panelDimensionTopY * 1.0f / sizeY;

            for (int y = 0; y < sizeY; y++)
                for (int x = 0; x < sizeX; x++)
                {
                    if (levelTop > 1)
                    {
                        int C = data[x, y, levelTop - 2];
                        if (C > 0)
                        {
                            SS.Color = colors1[C].ToColor();
                            G.FillRectangle(SS, 32 + x * cellX, y * cellY, cellX, cellY);
                            G.FillRectangle(KK, 32 + x * cellX, y * cellY, cellX, cellY);
                        }
                        if (C < 0)
                        {
                            SS.Color = colors1[-C].ToColor();
                            G.DrawRectangle(new Pen(SS.Color, cellX / 4), 32 + x * cellX + cellX / 8, y * cellY + cellX / 8, cellX - cellX / 4, cellY - cellX / 4);
                            G.DrawRectangle(new Pen(Color.FromArgb(180, 0, 0, 0), cellX / 4), 32 + x * cellX + cellX / 8, y * cellY + cellX / 8, cellX - cellX / 4, cellY - cellX / 4);
                        }
                    }


                    if (levelTop > 0)
                    {
                        int C = data[x, y, levelTop - 1];
                        if (C > 0)
                        {
                            SS.Color = colors1[C].ToColor();
                            G.FillRectangle(SS, 32 + x * cellX, y * cellY, cellX, cellY);
                            G.FillRectangle(KK2, 32 + x * cellX, y * cellY, cellX, cellY);
                        }
                        if (C < 0)
                        {
                            SS.Color = colors1[-C].ToColor();
                            G.DrawRectangle(new Pen(SS.Color, cellX / 4), 32 + x * cellX + cellX / 8, y * cellY + cellX / 8, cellX - cellX / 4, cellY - cellX / 4);
                            G.DrawRectangle(new Pen(Color.FromArgb(90, 0, 0, 0), cellX / 4), 32 + x * cellX + cellX / 8, y * cellY + cellX / 8, cellX - cellX / 4, cellY - cellX / 4);
                        }
                    }

                    int C2 = data[x, y, levelTop];
                    if (C2 > 0)
                    {
                        SS.Color = colors1[C2].ToColor();
                        G.FillRectangle(SS, 32 + x * cellX, y * cellY, cellX, cellY);
                    }
                    if (C2 < 0)
                    {
                        SS.Color = colors1[-C2].ToColor();
                        G.DrawRectangle(new Pen(SS.Color, cellX / 4), 32 + x * cellX + cellX / 8, y * cellY + cellX / 8, cellX - cellX / 4, cellY - cellX / 4);
                    }
                }
        }
        public void DrawSide(Graphics Gl)
        {
            Gl.SmoothingMode = SmoothingMode.HighSpeed;
            Gl.CompositingQuality = CompositingQuality.HighSpeed;
            Gl.InterpolationMode = InterpolationMode.NearestNeighbor;

            SolidBrush SS = new SolidBrush(Color.Black);
            SolidBrush KK = new SolidBrush(Color.FromArgb(180, 0, 0, 0));
            SolidBrush KK2 = new SolidBrush(Color.FromArgb(90, 0, 0, 0));

            float cellX = panelDimensionSideX * 1.0f / sizeX;
            float cellZ = panelDimensionSideY * 1.0f / sizeZ;

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                {

                    //draw pixels two levels below
                    if (levelSide > 1)
                    {
                        int C = data[x, levelSide - 2, z];
                        if (C > 0)
                        {
                            SS.Color = colors1[C].ToColor();
                            Gl.FillRectangle(SS, 32 + x * cellX, (sizeZ - 1 - z) * cellZ, cellX, cellZ);
                            Gl.FillRectangle(KK, 32 + x * cellX, (sizeZ - 1 - z) * cellZ, cellX, cellZ);
                        }
                        if (C < 0)
                        {
                            SS.Color = colors1[-C].ToColor();
                            Gl.DrawRectangle(new Pen(SS.Color, cellX / 4), 32 + x * cellX + cellX / 8, (sizeZ - 1 - z) * cellZ + cellX / 8, cellX - cellX / 4, cellZ - cellX / 4);
                            Gl.DrawRectangle(new Pen(Color.FromArgb(180, 0, 0, 0), cellX / 4), 32 + x * cellX + cellX / 8, (sizeZ - 1 - z) * cellZ + cellX / 8, cellX - cellX / 4, cellZ - cellX / 4);
                        }
                    }

                    //draw pixels one level below
                    if (levelSide > 0)
                    {
                        int C = data[x, levelSide - 1, z];
                        if (C > 0)
                        {
                            SS.Color = colors1[C].ToColor();
                            Gl.FillRectangle(SS, 32 + x * cellX, (sizeZ - 1 - z) * cellZ, cellX, cellZ);
                            Gl.FillRectangle(KK2, 32 + x * cellX, (sizeZ - 1 - z) * cellZ, cellX, cellZ);
                        }
                        if (C < 0)
                        {
                            SS.Color = colors1[-C].ToColor();
                            Gl.DrawRectangle(new Pen(SS.Color, cellX / 4), 32 + x * cellX + cellX / 8, (sizeZ - 1 - z) * cellZ + cellX / 8, cellX - cellX / 4, cellZ - cellX / 4);
                            Gl.DrawRectangle(new Pen(Color.FromArgb(90, 0, 0, 0), cellX / 4), 32 + x * cellX + cellX / 8, (sizeZ - 1 - z) * cellZ + cellX / 8, cellX - cellX / 4, cellZ - cellX / 4);
                        }
                    }

                    int C2 = data[x, levelSide, z];

                    //draw current pixel
                    if (C2 > 0)
                    {
                        SS.Color = colors1[C2].ToColor();
                        Gl.FillRectangle(SS, 32 + x * cellX, (sizeZ - 1 - z) * cellZ, cellX, cellZ);
                    }
                    if (C2 < 0)
                    {
                        SS.Color = colors1[-C2].ToColor();
                        Gl.DrawRectangle(new Pen(SS.Color, cellX / 4), 32 + x * cellX + cellX / 8, (sizeZ - 1 - z) * cellZ + cellX / 8, cellX - cellX / 4, cellZ - cellX / 4);
                    }
                }
        }

#endregion

#region files

        // SAVE AND LOAD
        public void Save2(string path)
        {
            string ggg = "";
            ggg += name + ";pallete;" + sizeX + ";" + sizeY + ";" + sizeZ + ";;;;;;";

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        ggg += data[x, y, z] + ",";
                    }

            ggg.TrimEnd(',');

            File.WriteAllText(path, ggg);
        }
        public void Save2()
        {
            string ggg = "";
            ggg += name + ";pallete;" + sizeX + ";" + sizeY + ";" + sizeZ + ";;;;;;";

            for (int z = 0; z < sizeZ; z++)
                for (int x = 0; x < sizeX; x++)
                    for (int y = 0; y < sizeY; y++)
                    {
                        ggg += data[x, y, z] + ",";
                    }

            ggg.TrimEnd(',');

            File.WriteAllText(currentFile, ggg);
        }
        public static TIcon Load2(string path)
        {
            TIcon TT = new TIcon();

            string ggg = File.ReadAllText(path);

            string[] www = ggg.Split(';');

            TT.currentFile = www[0];
            TT.name = www[0];
            TT.sizeX = Int16.Parse(www[2]);
            TT.sizeY = Int16.Parse(www[3]);
            TT.sizeZ = Int16.Parse(www[4]);
            TT.data = new int[TT.sizeX, TT.sizeY, TT.sizeZ];

            string[] kkk = www[10].Split(',');

            int ppp = 0;
            for (int z = 0; z < TT.sizeZ; z++)
                for (int x = 0; x < TT.sizeX; x++)
                    for (int y = 0; y < TT.sizeY; y++)
                    {
                        TT.data[x, y, z] = Int16.Parse(kkk[ppp]);
                        ppp++;
                    }

            return TT;
        }

#endregion


    }
}
